package kr.co.mkblog.mkblogopenglesexample;

import android.opengl.GLES30;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

/**
 * Created by mkblog on 2018-10-13
 */

public class mkGLExample {
    private String mkShaderCode = "";
    private String mkFragCode = "";
    private String mkSecondFragCode = "";

    //MK: Triangle Vertex
    private float[] mkFirstVertices = {
            0.0f, 0.0f, 0.0f,
            -0.5f, -0.5f, 0.0f,
            0.0f, -0.5f, 0.0f,
            0.5f, -0.5f, 0.0f
    };

    private int[] mkFirstIndices = {
            0, 1, 2,
            0, 3, 2
    };

    private float[] mkSecondVertices = {
            0.0f, 0.0f, 0.0f,
            -0.5f, 0.5f, 0.0f,
            0.0f, 0.5f, 0.0f,
            0.5f, 0.5f, 0.0f
    };

    private int[] mkSecondIndices = {
            0, 1, 2,
            0, 3, 2
    };

    private int mkVertexShader;
    private int mkFragShader;
    private int mkProgram;
    private int mkSecondProgram;

    private int[] mkVAO;
    private int[] mkVBO;
    private int[] mkEBO;


    public mkGLExample(){
        //MK: Shader Code
        mkShaderCode =      "#version 300 es \n"
                +   "layout (location = 0) in vec3 aPos; "
                +   " void main() {"
                +   "gl_Position = vec4(aPos.x, aPos.y, aPos.z, 1.0); }";
        //MK: Fragement Code
        mkFragCode =      "#version 300 es \n"
                +   "out highp vec4 FragColor;"
                +   "void main() {"
                +   "FragColor = vec4(1.0f, 0.5f, 0.2f, 1.0f);}";

        //MK: Fragement Code
        mkSecondFragCode =      "#version 300 es \n"
                +   "out highp vec4 FragColor;"
                +   "void main() {"
                +   "FragColor = vec4(0.1f, 0.1f, 0.1f, 1.0f);}";

        //MK: Shader Compile
        mkVertexShader = GLES30.glCreateShader(GLES30.GL_VERTEX_SHADER);
        GLES30.glShaderSource(mkVertexShader, mkShaderCode);
        GLES30.glCompileShader(mkVertexShader);

        //MK: Fragment Compile
        mkFragShader = GLES30.glCreateShader(GLES30.GL_FRAGMENT_SHADER);
        GLES30.glShaderSource(mkFragShader, mkFragCode);
        GLES30.glCompileShader(mkFragShader);

        //MK: Create GL Program and Attach Shader and Fragment Code
        mkProgram = GLES30.glCreateProgram();
        GLES30.glAttachShader(mkProgram, mkVertexShader);
        GLES30.glAttachShader(mkProgram, mkFragShader);
        GLES30.glLinkProgram(mkProgram);
        GLES30.glDeleteShader(mkFragShader);

        //MK: Second Fragment Compile
        mkFragShader = GLES30.glCreateShader(GLES30.GL_FRAGMENT_SHADER);
        GLES30.glShaderSource(mkFragShader, mkSecondFragCode);
        GLES30.glCompileShader(mkFragShader);

        //MK: Create GL Second Program and Attach Shader and Fragment Code
        mkSecondProgram = GLES30.glCreateProgram();
        GLES30.glAttachShader(mkSecondProgram, mkVertexShader); //MK: Delete 하지 않고 그냥 사용했는데 정상적으로 동작확인됨
        GLES30.glAttachShader(mkSecondProgram, mkFragShader);
        GLES30.glLinkProgram(mkSecondProgram);

        GLES30.glDeleteShader(mkVertexShader);
        GLES30.glDeleteShader(mkFragShader);

        //MK: Make VAO, VBO
        mkVAO = new int[2];
        GLES30.glGenVertexArrays(2, mkVAO, 0);

        mkVBO = new int[2];
        GLES30.glGenBuffers(2, mkVBO, 0);

        //MK: Make Element Buffer Object (EBO)
        mkEBO = new int[2];
        GLES30.glGenBuffers(2, mkEBO, 0);

        //MK: VAO Bind 이후에 Bind되는 VBO, EBO는 VAO에 저장되는 것으로 판단됨
        GLES30.glBindVertexArray(mkVAO[0]);

        //MK: VBO Bind
        GLES30.glBindBuffer(GLES30.GL_ARRAY_BUFFER, mkVBO[0]);

        //MK: Vertex 좌표를 mkVBO로 복사하는 과정
        ByteBuffer mkByteBuffer = ByteBuffer.allocateDirect(mkFirstVertices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer mkVertexBuffer = mkByteBuffer.asFloatBuffer();
        mkVertexBuffer.put(mkFirstVertices);
        mkVertexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ARRAY_BUFFER, mkFirstVertices.length*4, mkVertexBuffer, GLES30.GL_STATIC_DRAW );

        //MK: mkVBO의 Offset, Vec사이즈 등을 설정하는 부분
        GLES30.glVertexAttribPointer(0, 3, GLES30.GL_FLOAT, false, 3 * 4, 0);
        //MK: mkVBO가 한개 Bind되어서 0으로 설정한듯 함 (계속 공부를 하면서 알아볼 필요가 있음)
        GLES30.glEnableVertexAttribArray(0);

        GLES30.glBindBuffer(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkEBO[0]);
        mkByteBuffer =  ByteBuffer.allocateDirect(mkFirstIndices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        IntBuffer mkIndexBuffer = mkByteBuffer.asIntBuffer();
        mkIndexBuffer.put(mkFirstIndices);
        mkIndexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkFirstIndices.length * 4, mkIndexBuffer, GLES30.GL_STATIC_DRAW);

        //MK: Second Triangle Vertex
        GLES30.glBindVertexArray(mkVAO[1]);

        GLES30.glBindBuffer(GLES30.GL_ARRAY_BUFFER, mkVBO[1]);

        mkByteBuffer = ByteBuffer.allocateDirect(mkSecondVertices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        mkVertexBuffer = mkByteBuffer.asFloatBuffer();
        mkVertexBuffer.put(mkSecondVertices);
        mkVertexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ARRAY_BUFFER, mkSecondVertices.length*4, mkVertexBuffer, GLES30.GL_STATIC_DRAW );

        //MK: mkVBO의 Offset, Vec사이즈 등을 설정하는 부분
        GLES30.glVertexAttribPointer(0, 3, GLES30.GL_FLOAT, false, 3 * 4, 0);
        //MK: mkVBO가 한개 Bind되어서 0으로 설정한듯 함 (계속 공부를 하면서 알아볼 필요가 있음)
        GLES30.glEnableVertexAttribArray(0);

        GLES30.glBindBuffer(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkEBO[1]);

        mkByteBuffer =  ByteBuffer.allocateDirect(mkSecondIndices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        mkIndexBuffer = mkByteBuffer.asIntBuffer();
        mkIndexBuffer.put(mkSecondIndices);
        mkIndexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkSecondIndices.length * 4, mkIndexBuffer, GLES30.GL_STATIC_DRAW);

    }
    public void draw(){
        //MK: First DrawCall
        GLES30.glUseProgram(mkProgram);
        GLES30.glBindVertexArray(mkVAO[0]);
        GLES30.glDrawElements(GLES30.GL_TRIANGLES, 6, GLES30.GL_UNSIGNED_INT, 0);

        //MK: Second DrawCall
        GLES30.glUseProgram(mkSecondProgram);
        GLES30.glBindVertexArray(mkVAO[1]);
        GLES30.glDrawElements(GLES30.GL_TRIANGLES, 6, GLES30.GL_UNSIGNED_INT, 0);
    }
}